/**************************************************************************//**
 * @item     CosyOS-II Kernel
 * @file     sv_create.h
 * @brief    创建服务
 * @details  用于创建 CosyOS 内核对象。
 * @author   迟凯峰
 * @version  V3.1.0
 * @date     2024.11.11
 ******************************************************************************/

#ifndef __SV_CREATE_H
#define __SV_CREATE_H

/**
 @addtogroup CosyOS_内核服务
 @{
 *//**
 \defgroup   创建服务
 \brief      用于创建 CosyOS 内核对象。
 @{
 */

/*
 * 声明任务
 */

/** 声明静态任务：无私信 */
#define sCSV_ExternTask(name) \
	extern s_tasknode_tsp u_taskhandle_##name; \
	extern s_tasknode_ts u_taskhand_##name; \
	void name(void)

/** 声明动态任务：无私信 */
#define dCSV_ExternTask(name) \
	extern s_tasknode_tsp u_taskhandle_##name; \
	extern s_taskhand_ts u_taskhand_##name; \
	void name(void)

/** 声明静态任务：有私信 */
#define sCSV_ExternTask_Msg(name) \
	extern s_tasknode_tsp u_taskhandle_##name; \
	extern s_tasknode_ts u_taskhand_##name; \
	void name(MCUCFG_TASKMSG_VAR, 

/** 声明动态任务：有私信 */
#define dCSV_ExternTask_Msg(name) \
	extern s_tasknode_tsp u_taskhandle_##name; \
	extern s_taskhand_ts u_taskhand_##name; \
	void name(MCUCFG_TASKMSG_VAR, 



/*
 * 创建任务
 */

#if SYSCFG_DEBUGGING == __ENABLED__
	#if SYSCFG_SAFERUNTIME == __ENABLED__
		/** 静态创建任务：启用 DEBUGGING，启用 安全运行时 */
		#define sCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_ts u_taskhand_##name; \
			s_tasknode_tsp u_taskhandle_##name = &u_taskhand_##name; \
			MCUCFG_STACK_ALIGN static s_u8_t u_taskstack_##name[tss]; \
			s_tasknode_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent, #name, srt, u_taskstack_##name, 0}; \
			void name \
			sCreateTaskMsg_##ntm
	#else	/** 静态创建任务：启用 DEBUGGING，禁用 安全运行时 */
		#define sCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_ts u_taskhand_##name; \
			s_tasknode_tsp u_taskhandle_##name = &u_taskhand_##name; \
			MCUCFG_STACK_ALIGN static s_u8_t u_taskstack_##name[tss]; \
			s_tasknode_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent, #name, /**/ u_taskstack_##name, 0}; \
			void name \
			sCreateTaskMsg_##ntm
	#endif
#else
	#if SYSCFG_SAFERUNTIME == __ENABLED__
		/** 静态创建任务：禁用 DEBUGGING，启用 安全运行时 */
		#define sCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_ts u_taskhand_##name; \
			s_tasknode_tsp u_taskhandle_##name = &u_taskhand_##name; \
			MCUCFG_STACK_ALIGN static s_u8_t u_taskstack_##name[tss]; \
			s_tasknode_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent, /****/ srt, u_taskstack_##name, 0}; \
			void name \
			sCreateTaskMsg_##ntm
	#else	/** 静态创建任务：禁用 DEBUGGING，禁用 安全运行时 */
		#define sCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_ts u_taskhand_##name; \
			s_tasknode_tsp u_taskhandle_##name = &u_taskhand_##name; \
			MCUCFG_STACK_ALIGN static s_u8_t u_taskstack_##name[tss]; \
			s_tasknode_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent, /****/ /**/ u_taskstack_##name, 0}; \
			void name \
			sCreateTaskMsg_##ntm
	#endif
#endif

#if SYSCFG_DEBUGGING == __ENABLED__
	#if SYSCFG_SAFERUNTIME == __ENABLED__
		/** 动态创建任务：启用 DEBUGGING，启用 安全运行时 */
		#define dCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_tsp u_taskhandle_##name = OS_NULL; \
			s_taskhand_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent, #name, srt}; \
			void name \
			sCreateTaskMsg_##ntm
	#else	/** 动态创建任务：启用 DEBUGGING，禁用 安全运行时 */
		#define dCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_tsp u_taskhandle_##name = OS_NULL; \
			s_taskhand_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent, #name}; \
			void name \
			sCreateTaskMsg_##ntm
	#endif
#else
	#if SYSCFG_SAFERUNTIME == __ENABLED__
		/** 动态创建任务：禁用 DEBUGGING，启用 安全运行时 */
		#define dCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_tsp u_taskhandle_##name = OS_NULL; \
			s_taskhand_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent, srt}; \
			void name \
			sCreateTaskMsg_##ntm
	#else	/** 动态创建任务：禁用 DEBUGGING，禁用 安全运行时 */
		#define dCSV_CreateTask(name, pri, tss, srt, ntm, ent) \
			s_tasknode_tsp u_taskhandle_##name = OS_NULL; \
			s_taskhand_ts u_taskhand_##name = {0, 0, pri, tss, &u_taskhandle_##name, ent}; \
			void name \
			sCreateTaskMsg_##ntm
	#endif
#endif
/** 静态创建定时中断任务 */
#define sCSV_CreateTask_TimInt(tmid, arl, name, pri, tss, srt, ntm) \
	void name(void); \
	s_tasknode_ts u_taskhand_##name; \
	s_timinthand_ts _STATIC_MEM_ u_timinthand_##tmid = {0, 0, arl, 1, &u_taskhand_##name}; \
	sCSV_CreateTask(name, pri, tss, srt, ntm, name)

/** 动态创建定时中断任务 */
#define dCSV_CreateTask_TimInt(tmid, arl, name, pri, tss, srt, ntm) \
	void name(void); \
	s_taskhand_ts u_taskhand_##name; \
	s_timinthand_ts _STATIC_MEM_ u_timinthand_##tmid = {0, 0, arl, 1, &u_taskhand_##name}; \
	dCSV_CreateTask(name, pri, tss, srt, ntm, name)

/** 静态创建定时查询任务 */
#define sCSV_CreateTask_TimQry(tmid, event, arl, name, pri, tss, srt, ntm) \
	void name(void); \
	s_bool_t u_timqryevent_##tmid(void) MCUCFG_C51USING {return (event ? true : false);} \
	s_tasknode_ts u_taskhand_##name; \
	s_timqryhand_ts _STATIC_MEM_ u_timqryhand_##tmid = {(s_timqry_t)(SYSCFG_USERTIMQRYINIT - 1), (s_timqry_t)(SYSCFG_USERTIMQRYINIT - 1), arl, 1, &u_taskhand_##name, u_timqryevent_##tmid}; \
	sCSV_CreateTask(name, pri, tss, srt, ntm, name)

/** 动态创建定时查询任务 */
#define dCSV_CreateTask_TimQry(tmid, event, arl, name, pri, tss, srt, ntm) \
	void name(void); \
	s_bool_t u_timqryevent_##tmid(void) MCUCFG_C51USING {return (event ? true : false);} \
	s_taskhand_ts u_taskhand_##name; \
	s_timqryhand_ts _STATIC_MEM_ u_timqryhand_##tmid = {(s_timqry_t)(SYSCFG_USERTIMQRYINIT - 1), (s_timqry_t)(SYSCFG_USERTIMQRYINIT - 1), arl, 1, &u_taskhand_##name, u_timqryevent_##tmid}; \
	dCSV_CreateTask(name, pri, tss, srt, ntm, name)



/*
 * 创建钩子
 */

/** 创建定时中断钩子 */
#define sCSV_CreateHook_TimInt(tmid, arl, name) \
	void name(void); \
	s_timinthand_ts _STATIC_MEM_ u_timinthand_##tmid = {0, 0, arl, 0, name}; \
	void name(void) MCUCFG_C51USING

/** 创建定时查询钩子 */
#define sCSV_CreateHook_TimQry(tmid, event, arl, name) \
	s_bool_t u_timqryevent_##tmid(void) MCUCFG_C51USING {return (event ? true : false);} \
	void name(void); \
	s_timqryhand_ts _STATIC_MEM_ u_timqryhand_##tmid = {(s_timqry_t)(SYSCFG_USERTIMQRYINIT - 1), (s_timqry_t)(SYSCFG_USERTIMQRYINIT - 1), arl, 0, name, u_timqryevent_##tmid}; \
	void name(void) MCUCFG_C51USING



/*
 * 互斥信号量
 */

/** 声明互斥信号量 */
#define sCSV_ExternMut(name) \
	extern s_mutex_ts name

/** 创建互斥信号量 */
#define sCSV_CreateMut(name) \
	s_mutex_ts name = {false, OS_NULL, 0}



/*
 * 二值信号量
 */

/** 声明二值信号量 */
#define sCSV_ExternBin(name) \
	extern s_binary_ts name

/** 创建二值信号量 */
#define sCSV_CreateBin(name, init) \
	s_binary_ts name = {init, OS_NULL}



/*
 * 计数信号量
 */

/** 声明计数信号量 */
#define sCSV_ExternSem(name) \
	extern s_semaph_ts name

/** 创建计数信号量 */
#define sCSV_CreateSem(name, init, max) \
	s_semaph_ts name = {init, max, OS_NULL}



/*
 * 飞信
 */

/** 声明信箱 */
#define sCSV_ExternFetion(name) \
	extern s_tionbox_ts name

/** 创建信箱 */
#define sCSV_CreateFetion(name) \
	s_tionbox_ts name = {false, OS_NULL}



/*
 * 邮箱
 */

/** 声明邮箱 */
#define sCSV_ExternMailbox(name) \
	extern s_mailbox_ts name

/** 创建邮箱 */
#define sCSV_CreateMailbox(name) \
	s_mailbox_ts name = {false, OS_NULL, OS_NULL}



/*
 * 消息队列
 */

/** 声明静态队列 */
#define sCSV_ExternQueue(name) \
	extern ss_queue_ts name

/** 声明动态队列 */
#define dCSV_ExternQueue(name) \
	extern sd_queue_ts name

/** 创建静态队列 */
#define sCSV_CreateQueue(name, mode, len) \
	void *u_queue_##name[len]; \
	ss_queue_ts name = {0, __STATIC__, mode, len, OS_NULL, true, u_queue_##name, u_queue_##name + len - 1, u_queue_##name}

/** 创建动态队列 */
#define dCSV_CreateQueue(name, mode, len) \
	sd_queue_ts name = {0, __DYNAMIC__, mode, len, OS_NULL, true, OS_NULL, OS_NULL}



/*
 * 事件标志组（必须声明）
 */

/** 声明标志组 */
#define sCSV_ExternFlagGroup(name, bits) \
	typedef struct{bits}u_##name##_ts; \
	extern volatile u_##name##_ts name

/** 创建标志组 */
#define sCSV_CreateFlagGroup(name) \
	volatile u_##name##_ts name



/*
 * 线程内存
 */

/** 创建线程内存池 */
#define sCSV_CreateMempool \
	static s_thrmem_ts u_thrmem = {OS_NULL, OS_NULL, 0}

/** @} */
/** @} */

#endif
